@@ -1,9 +1,13 @@
-from math import ceil
-from typing import Callable
+from typing import Union,Callable,Optional,_GenericAlias,_VariadicGenericAlias
 
-def raise_type_error(var,name,types):
-	if type(types) is not tuple: types = (types,)
-	if var and not isinstance(var,types): raise TypeError(f"{name} must be {' or '.join([x.__name__ for x in types])}")
+def type_check(func):
+	def wrapper(*args,**kwargs):
+		args_list = [(args[i+1],v,t) if i+1 < len(args) else (kwargs.get(v),v,t) for i,(v,t) in enumerate(func.__annotations__.items())]
+		for a,v,t in args_list:
+			if (isinstance(t,_VariadicGenericAlias) and not isinstance(a,t)) or (isinstance(t,_GenericAlias) and t.__args__ and not isinstance(a,t.__args__)) or (not isinstance(t,(_GenericAlias,_VariadicGenericAlias)) and not isinstance(a,t)):
+				raise TypeError(f"{v} must be {(not isinstance(t,_GenericAlias) and t) or (t.__args__ and ' or '.join(x.__name__ for x in t.__args__)) or 'function'}")
+		return func(*args)
+	return wrapper
 
 class State:
 
@@ -24,41 +28,38 @@ class State:
 	def __str__(self):
 		return f'State <{self.name}>'
 
-	def add_keyboard(self,keyboard,size=None,max_row=3):
+	@type_check
+	def add_keyboard(self,keyboard:Union[list,dict],size:Optional[tuple]=None,max_row:Optional[int]=3):
 		'''Add inline keyboard for the state'''
-		raise_type_error(keyboard,'keyboard',(list,dict))
-		raise_type_error(size,'size',tuple)
 		if isinstance(keyboard,list): keyboard = dict(enumerate(keyboard))
 		if size and sum(size) != len(keyboard): raise ValueError(f'Keyboard length ({len(keyboard)}) must be the same size as the sum of row size ({sum(size)}).')
 		elem_n = len(keyboard)
 		if not size: size = [max_row for _ in range(elem_n//max_row)] + ([r,] if (r := elem_n % max_row) else [])
 		self.callback = (keyboard,tuple(size))
 
-	def add_text(self,regex=None,error=None):
+	@type_check
+	def add_text(self,regex:Optional[str]=None,error:Optional[str]=None):
 		'''Add handler for text input'''
-		raise_type_error(regex,'regex',str)
-		raise_type_error(error,'error',str)
 		if not regex: regex = r'^.*$' 
 		self.text = (regex,error)
 
-	def add_action(self,function):
+	@type_check
+	def add_action(self,function:Callable):
 		'''Add action to do with this command'''
-		raise_type_error(function,'function',(Callable))
 		self.action = function
 
-	def add_dynamic_keyboard(self,function,max_row=3):
+	@type_check
+	def add_dynamic_keyboard(self,function:Callable,max_row:Optional[int]=3):
 		'''Add action to build a keyboard dynamically'''
-		raise_type_error(function,'function',(Callable))
-		raise_type_error(max_row,'max_row',(int))
 		self.build = function
 		self.max_row = max_row
 
-	def add_custom_keyboard(self,function):
+	@type_check
+	def add_custom_keyboard(self,function:Callable):
 		'''Add function to build custom ReplyMarkup keyboard | must return a value for ReplyMarkup'''
-		raise_type_error(function,'function',(Callable))
 		self.custom = function
 
+	@type_check
 	def add_dynamic_routes(self,function):
 		'''Add function to create dynamic routes | must return 3 value (routes,default,back)'''
-		raise_type_error(function,'function',(Callable))
 		self.routes = function
